import typing

import proto_compile.versions as versions
from proto_compile.utils import PathLike
from proto_compile.versions import Target


class BaseCompilerOptions:
    def __init__(
        self,
        proto_source_dir: PathLike,
        output_dir: PathLike,
        minimal_include_dir: bool = False,
        clear_output_dirs: bool = False,
        verbosity: typing.Optional[int] = None,
        protoc_version: typing.Optional[str] = None,
    ) -> None:
        self.proto_source_dir = proto_source_dir
        self.output_dir = output_dir
        self.minimal_include_dir = minimal_include_dir
        self.clear_output_dirs = clear_output_dirs
        self.verbosity = verbosity or 0
        self.protoc_version = protoc_version or versions.DEFAULT_PROTOC_VERSION


class CompileTarget:
    def __init__(
        self,
        language: Target,
        out_options: typing.Optional[str] = None,
        output_dir: typing.Optional[PathLike] = None,
        plugin_version: typing.Optional[str] = None,
    ):
        self.language = language
        self.out_options = out_options
        self.output_dir = output_dir
        self.plugin_version = plugin_version


class CompilerOptions:
    def __init__(
        self, base_options: BaseCompilerOptions, targets: typing.List[CompileTarget],
    ) -> None:
        self.proto_source_dir = base_options.proto_source_dir
        self.minimal_include_dir = base_options.minimal_include_dir
        self.clear_output_dirs = base_options.clear_output_dirs
        self.output_dir = base_options.output_dir
        self.verbosity = base_options.verbosity
        self.protoc_version = (
            base_options.protoc_version or versions.DEFAULT_PROTOC_VERSION
        )
        self.targets = targets



# -*- coding: utf-8 -*-

"""Main module."""

import os
import platform
import shutil
import subprocess
import tempfile
import typing

from proto_compile import versions as versions
from proto_compile.options import BaseCompilerOptions, CompilerOptions, CompileTarget
from proto_compile.plugins import PLUGINS, PROTOC_RELEASE_BASE_URL, ProtoCompiler
from proto_compile.utils import PathLike, download_executable, print_command, rglob
from proto_compile.versions import Target


class DefaultProtoCompiler(ProtoCompiler):
    def __init__(self, executable: PathLike) -> None:
        self.executable = executable

    def compile(self, arguments: typing.List[str], verbosity: int = 0) -> None:
        protoc_command_str = str(" ").join([str(self.executable)] + arguments)
        if verbosity > 0:
            print(protoc_command_str)
        print_command(
            protoc_command_str,
            stderr=subprocess.STDOUT,
            shell=True,
            verbosity=verbosity,
        )


def compile(options: CompilerOptions) -> None:
    abs_source = os.path.abspath(options.proto_source_dir)

    proto_files = rglob(abs_source, match="*.proto", absolute=True)
    if not len(proto_files) > 0:
        print("{} does not contain any .proto files. Skipping...".format(abs_source))
        return

    if options.minimal_include_dir:
        abs_source = os.path.abspath(os.path.dirname(os.path.commonpath(proto_files)))

    tmp_dir = tempfile.mkdtemp()
    try:
        system = platform.system().lower()  # darwin
        system_alias = "osx" if system == "darwin" else system  # osx for darwin
        system_alias = "win64" if system == "windows" else system_alias  # windows
        machine_arch = "" if system == "windows" else platform.machine()

        if options.verbosity > 0:
            print(system_alias)
        protoc_release_url = PROTOC_RELEASE_BASE_URL
        protoc_release_url += "/v" + options.protoc_version
        protoc_release_url += (
            "/protoc-"
            + options.protoc_version
            + "-"
            + system_alias
            + ("" if system == "windows" else "-")
            + machine_arch
            + ".zip"
        )

        protoc_executable = download_executable(
            "protoc",
            url=protoc_release_url,
            executable="protoc/bin/protoc",
            dest_dir=tmp_dir,
            verbosity=options.verbosity,
        )
        print_command(
            "ls -la " + tmp_dir,
            stderr=subprocess.STDOUT,
            shell=True,
            verbosity=options.verbosity,
        )

        # eventually clear the output dirs
        # also find the correct compiler
        proto_compiler: ProtoCompiler = DefaultProtoCompiler(protoc_executable)
        for target in options.targets:
            abs_output = os.path.abspath(target.output_dir or options.output_dir)
            if os.path.exists(abs_output) and options.clear_output_dirs:
                shutil.rmtree(abs_output, ignore_errors=True)

        # construct protoc compiler command
        proto_arguments: typing.List[str] = ["-I={}".format(abs_source)] + [
            str(f) for f in proto_files
        ]

        for target in options.targets:
            abs_output = os.path.abspath(target.output_dir or options.output_dir)
            language = str(target.language.value)

            # get the required plugin
            if target.language in PLUGINS:
                plugin = PLUGINS[target.language]
                if plugin.compiler is not None:
                    proto_compiler = plugin.compiler

                try:
                    # attempt to install the plugin
                    plugin_executable = plugin.installed()
                    if plugin_executable is None:
                        plugin_executable = plugin.install(
                            tmp_dir,
                            version=target.plugin_version,
                            verbosity=options.verbosity,
                        )
                    if plugin_executable is not None:
                        proto_arguments.append(
                            "--plugin=protoc-gen-{}={}".format(
                                language, plugin_executable,
                            )
                        )

                except NotImplementedError:
                    # show installation hints
                    try:
                        print(plugin.install_hint())
                    except NotImplementedError:
                        pass

            proto_arguments.append(
                "--{}_out={}{}".format(
                    str(target.language.value),
                    str((target.out_options + ":") if target.out_options else ""),
                    abs_output,
                )
            )

            # make sure the output path exists
            if not os.path.exists(abs_output):
                os.makedirs(abs_output)

        proto_compiler.compile(proto_arguments, verbosity=options.verbosity)
    finally:
        # Remove temporary directory
        shutil.rmtree(tmp_dir, ignore_errors=True)


def compile_grpc_web(
    options: BaseCompilerOptions,
    js_out_options: typing.Optional[str] = "import_style=commonjs,binary",
    js_output_dir: typing.Optional[PathLike] = None,
    grpc_web_out_options: typing.Optional[
        str
    ] = "import_style=typescript,mode=grpcwebtext",
    grpc_web_plugin_version: typing.Optional[str] = versions.DEFAULT_PLUGIN_VERSIONS[
        Target.GRPC_WEB
    ],
    grpc_web_output_dir: typing.Optional[PathLike] = None,
) -> None:
    return compile(
        CompilerOptions(
            base_options=options,
            targets=[
                CompileTarget(
                    Target.JAVASCRIPT,
                    output_dir=js_output_dir,
                    out_options=js_out_options,
                ),
                CompileTarget(
                    Target.GRPC_WEB,
                    out_options=grpc_web_out_options,
                    output_dir=grpc_web_output_dir,
                    plugin_version=grpc_web_plugin_version,
                ),
            ],
        )
    )


def compile_python_grpc(
    options: BaseCompilerOptions,
    py_out_options: typing.Optional[str] = None,
    py_output_dir: typing.Optional[PathLike] = None,
    py_grpc_out_options: typing.Optional[str] = None,
    py_grpc_output_dir: typing.Optional[PathLike] = None,
) -> None:
    return compile(
        CompilerOptions(
            base_options=options,
            targets=[
                CompileTarget(
                    Target.PYTHON, output_dir=py_output_dir, out_options=py_out_options,
                ),
                CompileTarget(
                    Target.PYTHON_GRPC,
                    out_options=py_grpc_out_options,
                    output_dir=py_grpc_output_dir,
                ),
            ],
        )
    )



# -*- coding: utf-8 -*-

"""Console script for proto_compile."""
import os
import sys
import typing

import click

import proto_compile.proto_compile as compiler
import proto_compile.versions as versions
from proto_compile.options import BaseCompilerOptions
from proto_compile.utils import PathLike
from proto_compile.versions import Target


def assert_valid_dir(
    ctx: click.core.Context, param: click.core.Parameter, value: str
) -> str:
    if os.path.exists(value) and os.path.isdir(value):
        return value
    raise click.BadParameter(
        'Directory "' + value + '" does not exist.', ctx=ctx, param=param
    )


def optional_assert_valid_dir(
    ctx: click.core.Context, param: click.core.Parameter, value: str
) -> str:
    if value == "":
        return value
    return assert_valid_dir(ctx, param, value)


base_proto_parent_dir_help = (
    "base proto parent dir used for protoc -I=<base_proto_parent_dir>. ",
    "Must be a valid directory that contains the proto files in <proto_source_dir>",
)


@click.group()
@click.argument("proto-source-dir", callback=assert_valid_dir, type=click.Path())
@click.argument("output-dir", type=click.Path())
@click.option(
    "--minimal-include-dir",
    default=False,
    type=bool,
    help=str(
        "determine the shortest proto source include path that contains all protos in the given source dir"
    ),
)
@click.option(
    "--clear-output-dirs",
    "-clear",
    is_flag=True,
    default=False,
    help=str("whether to clear the output directories before compilation"),
)
@click.option(
    "--verbosity",
    default=0,
    help=str("level of verbosity when printing to stdout (the higher the more output)"),
)
@click.option(
    "--protoc-version",
    default=versions.DEFAULT_PROTOC_VERSION,
    help="protoc version to use (default is %s)" % versions.DEFAULT_PROTOC_VERSION,
)
@click.pass_context
def proto_compile(
    ctx: click.Context,
    proto_source_dir: str,
    output_dir: str,
    minimal_include_dir: bool,
    clear_output_dirs: bool,
    verbosity: int,
    protoc_version: str,
) -> None:
    ctx.ensure_object(dict)
    ctx.obj["COMPILER_OPTIONS"] = BaseCompilerOptions(
        proto_source_dir=proto_source_dir,
        output_dir=output_dir,
        minimal_include_dir=minimal_include_dir,
        clear_output_dirs=clear_output_dirs,
        verbosity=verbosity,
        protoc_version=protoc_version,
    )


@proto_compile.command()
@click.option(
    "--js_out_options",
    default="import_style=commonjs,binary",
    help=str("options for the javascript proto compiler"),
)
@click.option(
    "--grpc_web_out_options",
    default="import_style=typescript,mode=grpcwebtext",
    help=str("options for the grpc web proto compiler"),
)
@click.option(
    "--grpc_web_plugin_version",
    default=versions.DEFAULT_PLUGIN_VERSIONS[Target.GRPC_WEB],
    help="grpc web plugin version to use (default is %s)"
    % versions.DEFAULT_PLUGIN_VERSIONS[Target.GRPC_WEB],
)
@click.pass_context
def grpc_web(
    ctx: click.Context,
    js_out_options: str,
    grpc_web_out_options: str,
    grpc_web_plugin_version: str,
) -> int:
    """ compile using the grpc-web preset """
    try:
        compiler.compile_grpc_web(
            options=ctx.obj["COMPILER_OPTIONS"],
            js_out_options=js_out_options,
            grpc_web_out_options=grpc_web_out_options,
            grpc_web_plugin_version=grpc_web_plugin_version,
        )
    except Exception as e:  # pragma: no cover
        raise click.ClickException(str(e))
    return 0


@proto_compile.command()
@click.option(
    "--py_out_options", default=None, help=str("options for the python proto compiler"),
)
@click.option(
    "--py_output_dir",
    default=None,
    help=str("separate output dir for the python generated files"),
)
@click.option(
    "--py_grpc_out_options",
    default=None,
    help=str("options for the python grpc proto compiler"),
)
@click.option(
    "--py_grpc_output_dir",
    default=None,
    help=str("separate output dir for the python grpc generated files"),
)
@click.pass_context
def python_grpc(
    ctx: click.Context,
    py_out_options: typing.Optional[str],
    py_output_dir: typing.Optional[PathLike],
    py_grpc_out_options: typing.Optional[str],
    py_grpc_output_dir: typing.Optional[PathLike],
) -> int:
    """ compile using the python grpc preset """
    try:
        compiler.compile_python_grpc(
            options=ctx.obj["COMPILER_OPTIONS"],
            py_out_options=py_out_options,
            py_output_dir=py_output_dir,
            py_grpc_out_options=py_grpc_out_options,
            py_grpc_output_dir=py_grpc_output_dir,
        )
    except Exception as e:  # pragma: no cover
        raise click.ClickException(str(e))
    return 0


if __name__ == "__main__":
    sys.exit(proto_compile(obj=dict()))  # pragma: no cover



# -*- coding: utf-8 -*-

"""Top-level package for proto-compile."""

__author__ = """romnn"""
__email__ = "contact@romnn.com"
__version__ = "0.1.6"

from proto_compile.options import BaseCompilerOptions, CompilerOptions, CompileTarget
from proto_compile.proto_compile import compile, compile_grpc_web, compile_python_grpc
from proto_compile.versions import Target

__all__ = [
    "compile",
    "compile_grpc_web",
    "compile_python_grpc",
    "Target",
    "BaseCompilerOptions",
    "CompileTarget",
    "CompilerOptions",
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `proto_compile` package."""

import os
import tempfile
import typing

import pytest
from click.testing import CliRunner

from proto_compile import cli, proto_compile
from proto_compile.options import (
    BaseCompilerOptions,
    CompilerOptions,
    CompileTarget,
    Target,
)
from proto_compile.utils import PathLike, rglob

TEST_DIR = os.path.dirname(os.path.realpath(__file__))
PROTO_DIR = os.path.join(TEST_DIR, "protos")


def sort(paths: typing.List[PathLike]) -> typing.List[str]:
    return sorted([str(p) for p in paths])


def base_options(
    out_dir: PathLike,
    proto_source_dir: PathLike = PROTO_DIR,
    clear_output_dirs: bool = True,
    verbosity: int = 3,
) -> BaseCompilerOptions:
    return BaseCompilerOptions(
        proto_source_dir=proto_source_dir,
        output_dir=out_dir,
        clear_output_dirs=clear_output_dirs,
        verbosity=verbosity,
    )


def test_command_line_interface() -> None:
    """Test the CLI."""
    runner = CliRunner()
    result = runner.invoke(cli.proto_compile)
    assert result.exit_code == 0
    assert "Usage: proto-compile" in result.output
    help_result = runner.invoke(cli.proto_compile, ["--help"])
    assert help_result.exit_code == 0
    assert "Show this message and exit." in help_result.output


def test_compile(subtests: typing.Any) -> None:
    """ Test compiling protos for different languages """
    test_targets: typing.List[
        typing.Tuple[typing.List[CompileTarget], typing.List[PathLike],]
    ] = [
        (
            [CompileTarget(Target.JAVASCRIPT)],
            [
                "routesummary.js",
                "point.js",
                "routenote.js",
                "feature.js",
                "rectangle.js",
            ],
        ),
        (
            [CompileTarget(Target.CPP)],
            ["example_service.pb.h", "example_service.pb.cc"],
        ),
        ([CompileTarget(Target.CSHARP)], ["ExampleService.cs"]),
        (
            [CompileTarget(Target.GO)],
            [
                "google.golang.org/grpc/examples/route_guide/routeguide/example_service.pb.go"
            ],
        ),
        (
            [CompileTarget(Target.JAVA)],
            [
                "io/grpc/examples/routeguide/PointOrBuilder.java",
                "io/grpc/examples/routeguide/Point.java",
                "io/grpc/examples/routeguide/RouteNoteOrBuilder.java",
                "io/grpc/examples/routeguide/Feature.java",
                "io/grpc/examples/routeguide/RectangleOrBuilder.java",
                "io/grpc/examples/routeguide/RouteNote.java",
                "io/grpc/examples/routeguide/RouteSummary.java",
                "io/grpc/examples/routeguide/FeatureOrBuilder.java",
                "io/grpc/examples/routeguide/RouteSummaryOrBuilder.java",
                "io/grpc/examples/routeguide/Rectangle.java",
                "io/grpc/examples/routeguide/RouteGuideProto.java",
            ],
        ),
        (
            [CompileTarget(Target.JAVA), CompileTarget(Target.KOTLIN)],
            [
                "io/grpc/examples/routeguide/PointOrBuilder.java",
                "io/grpc/examples/routeguide/Point.java",
                "io/grpc/examples/routeguide/RouteNoteOrBuilder.java",
                "io/grpc/examples/routeguide/Feature.java",
                "io/grpc/examples/routeguide/RectangleOrBuilder.java",
                "io/grpc/examples/routeguide/RouteSummaryKt.kt",
                "io/grpc/examples/routeguide/RouteNote.java",
                "io/grpc/examples/routeguide/RouteSummary.java",
                "io/grpc/examples/routeguide/RectangleKt.kt",
                "io/grpc/examples/routeguide/RouteGuideProtoKt.kt",
                "io/grpc/examples/routeguide/FeatureOrBuilder.java",
                "io/grpc/examples/routeguide/RouteSummaryOrBuilder.java",
                "io/grpc/examples/routeguide/FeatureKt.kt",
                "io/grpc/examples/routeguide/Rectangle.java",
                "io/grpc/examples/routeguide/RouteNoteKt.kt",
                "io/grpc/examples/routeguide/PointKt.kt",
                "io/grpc/examples/routeguide/RouteGuideProto.java",
            ],
        ),
        ([CompileTarget(Target.PYTHON)], ["example_service_pb2.py"]),
        (
            [
                CompileTarget(
                    Target.JAVASCRIPT, out_options="import_style=commonjs,binary",
                ),
                CompileTarget(
                    Target.GRPC_WEB,
                    out_options="import_style=typescript,mode=grpcwebtext",
                ),
            ],
            [
                "example_service_pb.d.ts",
                "example_service_pb.js",
                "Example_serviceServiceClientPb.ts",
            ],
        ),
        (
            [CompileTarget(Target.JAVASCRIPT), CompileTarget(Target.JAVASCRIPT_GRPC)],
            [
                "routesummary.js",
                "point.js",
                "routenote.js",
                "feature.js",
                "example_service_grpc_pb.js",
                "rectangle.js",
            ],
        ),
        (
            [
                CompileTarget(Target.TYPESCRIPT),
                CompileTarget(Target.JAVASCRIPT),
                CompileTarget(Target.JAVASCRIPT_GRPC),
            ],
            [
                "routesummary.js",
                "point.js",
                "routenote.js",
                "feature.js",
                "example_service_grpc_pb.js",
                "example_service_pb.d.ts",
                "rectangle.js",
            ],
        ),
        (
            [CompileTarget(Target.GO), CompileTarget(Target.GO_GRPC)],
            [
                "google.golang.org/grpc/examples/route_guide/routeguide/example_service.pb.go",
                "google.golang.org/grpc/examples/route_guide/routeguide/example_service_grpc.pb.go",
            ],
        ),
        (
            [
                CompileTarget(Target.GO, out_options="paths=source_relative"),
                CompileTarget(Target.GO_GRPC, out_options="paths=source_relative"),
            ],
            ["example_service.pb.go", "example_service_grpc.pb.go",],
        ),
        (
            [CompileTarget(Target.PYTHON), CompileTarget(Target.PYTHON_GRPC)],
            ["example_service_pb2.py", "example_service_pb2_grpc.py"],
        ),
    ]
    if os.environ.get("CI", False):
        test_targets += [
            ([CompileTarget(Target.DART)], []),
            ([CompileTarget(Target.MYPY)], []),
        ]
    for targets, outputs in test_targets:
        with subtests.test(
            msg="compiling {}".format("+".join([t.language.value for t in targets])),
            targets=[t.language.value for t in targets],
        ):
            with tempfile.TemporaryDirectory() as tmp:
                settings = CompilerOptions(
                    base_options=base_options(tmp), targets=targets,
                )
                proto_compile.compile(settings)
                assert sort(rglob(tmp)) == sort(outputs)

            with tempfile.TemporaryDirectory() as tmp1:
                with tempfile.TemporaryDirectory() as tmp2:
                    for target in targets:
                        target.output_dir = tmp2
                    options = CompilerOptions(
                        base_options=base_options(tmp1), targets=targets,
                    )
                    proto_compile.compile(options)
                    assert rglob(tmp1) == []
                    assert sort(rglob(tmp2)) == sort(outputs)



import fnmatch
import os
import subprocess
import typing

PathLike = typing.Union[str, os.PathLike]


def print_command(
    *cmd_args: typing.Any, verbosity: int = 0, **cmd_kwargs: typing.Any
) -> None:
    try:
        output = subprocess.check_output(*cmd_args, **cmd_kwargs).decode("utf-8")
        if verbosity > 1:
            print(output)
    except subprocess.CalledProcessError as e:  # pragma: no cover
        print(e.returncode)
        print(e.output)
        raise


def quote(s: str) -> str:
    return '"' + s + '"'


def download_executable(
    name: str, url: str, executable: PathLike, dest_dir: PathLike, verbosity: int = 0
) -> str:
    filename = os.path.basename(url)
    is_zip = os.path.splitext(filename)[1].lower() == ".zip"
    download_command = str(" ").join(
        [
            "wget",
            "-O",
            os.path.join(dest_dir, filename if is_zip else executable),
            quote(url),
        ]
    )
    if verbosity > 0:
        print(download_command)
    print_command(
        download_command, stderr=subprocess.STDOUT, shell=True, verbosity=verbosity,
    )
    if is_zip:
        unzip_command = str(" ").join(
            [
                "unzip",
                os.path.join(dest_dir, filename),
                "-d",
                os.path.join(dest_dir, name),
            ]
        )
        if verbosity > 0:
            print(unzip_command)
        print_command(
            unzip_command, stderr=subprocess.STDOUT, shell=True, verbosity=verbosity,
        )
    executable_path = os.path.join(dest_dir, executable)
    chmod_command = str(" ").join(["chmod", "+x", executable_path])
    if verbosity > 0:
        print(chmod_command)
    print_command(
        chmod_command, stderr=subprocess.STDOUT, shell=True, verbosity=verbosity
    )
    return executable_path


def rglob(
    folder: PathLike, absolute: bool = False, match: str = "*"
) -> typing.List[PathLike]:
    matches: typing.List[PathLike] = []
    for root, dirnames, filenames in os.walk(str(folder)):
        for filename in fnmatch.filter(filenames, match):
            abs_match = os.path.join(root, filename)
            matches.append(
                abs_match if absolute else os.path.relpath(abs_match, folder)
            )
    return matches


def executable_in_path(
    name: str, path: typing.Optional[typing.List[PathLike]] = None
) -> typing.Optional[PathLike]:
    """ Check if PATH contains an executable """
    path_items = path or os.environ["PATH"].split(os.pathsep)
    for prefix in path_items:
        filename = os.path.abspath(os.path.join(str(prefix), name))
        executable = os.access(filename, os.X_OK)
        is_not_directory = os.path.isfile(filename)
        if executable and is_not_directory:
            return filename
    return None



import platform
import subprocess
import typing

import pkg_resources
from grpc_tools.protoc import main as _compile_python_grpc

from proto_compile.utils import PathLike, download_executable, executable_in_path
from proto_compile.versions import DEFAULT_PLUGIN_VERSIONS, Target

PROTOC_RELEASE_BASE_URL = (
    "https://github.com/protocolbuffers/protobuf/releases/download"
)
GRPC_WEB_PLUGIN_RELEASE_BASE_URL = "https://github.com/grpc/grpc-web/releases/download"


class ProtoCompiler:
    def compile(self, arguments: typing.List[str], verbosity: int = 0) -> None:
        raise NotImplementedError()


class ProtocPlugin:
    def __init__(self,) -> None:
        self.executable = ""
        self.compiler: typing.Optional[ProtoCompiler] = None

    def installed(self) -> typing.Optional[PathLike]:
        return executable_in_path(self.executable)

    def install(
        self,
        dest_dir: PathLike,
        version: typing.Optional[str] = None,
        verbosity: int = 0,
    ) -> str:
        pass

    def install_hint(self) -> str:
        pass


class DartPlugin(ProtocPlugin):
    def __init__(self) -> None:
        super().__init__()
        self.executable = "protoc-gen-dart"

    def install_hint(self) -> str:
        return """
        The dart plugin has to be installed. Run:

            $ pub global activate protoc_plugin

        or consult the official documentation.
        """


class MyPyPlugin(ProtocPlugin):
    def __init__(self) -> None:
        super().__init__()
        self.executable = "protoc-gen-mypy"

    def install_hint(self) -> str:
        return """
        The {} plugin has to be installed. Run:

            $ pip install mypy-protobuf

        This is only necessary to use the generated files:

            $ mypy>=0.910 types-protobuf>=0.1.14

        or consult the official documentation.
        """.format(
            self.executable
        )


class PythonGrpcProtoCompiler(ProtoCompiler):
    def compile(self, arguments: typing.List[str], verbosity: int = 0) -> None:
        proto_include = pkg_resources.resource_filename("grpc_tools", "_proto")
        arguments = [""] + arguments + ["-I{}".format(proto_include)]
        command = "python -m grpc_tools.protoc {}".format(
            " ".join([str(arg) for arg in arguments[1:]])
        )

        if verbosity > 0:
            print(command)
        return_code = int(_compile_python_grpc(arguments))
        if return_code != 0:
            raise subprocess.CalledProcessError(cmd=command, returncode=return_code)


class PythonGrpcPlugin(ProtocPlugin):
    def __init__(self) -> None:
        super().__init__()
        self.executable = "grpc_node_plugin"
        self.compiler = PythonGrpcProtoCompiler()


class TypescriptPlugin(ProtocPlugin):
    def __init__(self) -> None:
        super().__init__()
        self.executable = "grpc_node_plugin"

    def install_hint(self) -> str:
        return """
        The {} plugin has to be installed. Run:

            $ npm install -g ts-protoc-gen

        or consult the official documentation.
        """.format(
            self.executable
        )


class GolangPlugin(ProtocPlugin):
    def __init__(self) -> None:
        super().__init__()
        self.executable = "protoc-gen-go"

    def install_hint(self) -> str:
        return """
        The {} plugin has to be installed. Run:

            $ go install google.golang.org/protobuf/cmd/protoc-gen-go
            $ export PATH="$PATH:$(go env GOPATH)/bin"

        or consult the official documentation.
        """.format(
            self.executable
        )


class GolangGrpcPlugin(ProtocPlugin):
    def __init__(self) -> None:
        super().__init__()
        self.executable = "protoc-gen-go-grpc"

    def install_hint(self) -> str:
        return """
        The {} plugin has to be installed. Run:

            $ go install google.golang.org/protobuf/cmd/protoc-gen-go
            $ go install google.golang.org/grpc/cmd/protoc-gen-go-grpc
            $ export PATH="$PATH:$(go env GOPATH)/bin"

        or consult the official documentation.
        """.format(
            self.executable
        )


class PHPGrpcPlugin(ProtocPlugin):
    def __init__(self) -> None:
        super().__init__()
        self.executable = "grpc_php_plugin"

    def install_hint(self) -> str:
        return """
        The {} plugin has to be installed. Run:

            $ pecl install grpc

        or consult the official documentation.
        """.format(
            self.executable
        )


class JavascriptGrpcPlugin(ProtocPlugin):
    def __init__(self) -> None:
        super().__init__()
        self.executable = "grpc_tools_node_protoc_plugin"

    def install_hint(self) -> str:
        return """
        The {} plugin has to be installed. Run:

            $ npm install -g grpc-tools

        or consult the official documentation.
        """.format(
            self.executable
        )


class GrpcWebPlugin(ProtocPlugin):
    def __init__(self) -> None:
        super().__init__()
        self.executable = "protoc-gen-grpc-web"

    def install_hint(self) -> str:
        raise NotImplementedError()

    def install(
        self,
        dest_dir: PathLike,
        version: typing.Optional[str] = None,
        verbosity: int = 0,
    ) -> str:
        system = platform.system().lower()  # darwin
        system_alias = "osx" if system == "darwin" else system  # osx for darwin
        system_alias = "win64" if system == "windows" else system_alias  # windows
        machine_arch = "x86_64" if system == "windows" else platform.machine()

        grpc_web_plugin_version = version or DEFAULT_PLUGIN_VERSIONS[Target.GRPC_WEB]
        grpc_web_plugin_release_url = GRPC_WEB_PLUGIN_RELEASE_BASE_URL
        grpc_web_plugin_release_url += "/" + grpc_web_plugin_version
        grpc_web_plugin_release_url += (
            "/protoc-gen-grpc-web-"
            + grpc_web_plugin_version
            + "-"
            + system
            + "-"
            + machine_arch
            + (".exe" if system == "windows" else "")
        )
        plugin_executable = download_executable(
            self.executable,
            url=grpc_web_plugin_release_url,
            executable=self.executable,
            dest_dir=dest_dir,
            verbosity=verbosity,
        )
        return plugin_executable


PLUGINS = {
    Target.DART: DartPlugin(),
    Target.MYPY: MyPyPlugin(),
    Target.TYPESCRIPT: TypescriptPlugin(),
    Target.GO: GolangPlugin(),
    # GRPC
    Target.JAVASCRIPT_GRPC: JavascriptGrpcPlugin(),
    Target.PYTHON_GRPC: PythonGrpcPlugin(),
    Target.GO_GRPC: GolangGrpcPlugin(),
    Target.GRPC_WEB: GrpcWebPlugin(),
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import find_packages, setup
import os

short_description = "No description has been added so far."

version = "0.1.6"

PROJECT_ROOT = os.path.dirname(os.path.realpath(__file__))

try:
    readme_rst = os.path.join(PROJECT_ROOT, "README.rst")
    readme_md = os.path.join(PROJECT_ROOT, "README.md")
    if os.path.isfile(readme_rst):
        with open(readme_rst) as readme_file:
            long_description = readme_file.read()
    elif os.path.isfile(readme_md):
        import m2r

        long_description = m2r.parse_from_file(readme_md)
    else:
        raise AssertionError("No readme file")
except (ImportError, AssertionError):
    long_description = short_description

requirements = ["Click>=6.0", "grpcio-tools"]
test_requirements = [
    "tox",
    "pytest",
    "pytest-cov",
    "pytest-xdist",
    "pytest-sugar",
    "mypy",
    "pyfakefs",
    "pytest-subtests",
    "types-setuptools",
]
coverage_requirements = ["coverage"]
formatting_requirements = ["flake8", "black==19.10b0", "isort"]
tool_requirements = [
    "m2r",
    "invoke",
    "pre-commit",
    "cookiecutter",
    "bump2version",
]
dev_requirements = (
    requirements
    + test_requirements
    + coverage_requirements
    + formatting_requirements
    + tool_requirements
)

setup(
    author="romnn",
    author_email="contact@romnn.com",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Environment :: Console",
        "Operating System :: OS Independent",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    entry_points={"console_scripts": ["proto-compile=proto_compile.cli:proto_compile"]},
    python_requires=">=3.6",
    install_requires=requirements,
    setup_requires=tool_requirements,
    tests_require=test_requirements,
    extras_require=dict(dev=dev_requirements, test=test_requirements),
    license="MIT",
    description=short_description,
    long_description=long_description,
    include_package_data=True,
    package_data={"proto_compile": []},
    keywords="proto-compile",
    name="proto-compile",
    packages=find_packages(include=["proto_compile"]),
    test_suite="tests",
    url="https://github.com/romnn/proto-compile",
    version=version,
    zip_safe=False,
)



import enum


class Target(enum.Enum):
    JAVASCRIPT = "js"
    TYPESCRIPT = "ts"
    JAVASCRIPT_GRPC = "grpc"
    CPP = "cpp"
    CSHARP = "csharp"
    DART = "dart"
    GO = "go"
    GO_GRPC = "go-grpc"
    JAVA = "java"
    KOTLIN = "kotlin"
    PYTHON = "python"
    PYTHON_GRPC = "grpc_python"
    MYPY = "mypy"
    GRPC_WEB = "grpc-web"


DEFAULT_PROTOC_VERSION = "3.17.3"

DEFAULT_PLUGIN_VERSIONS = {
    Target.GRPC_WEB: "1.2.0",
}
